@@ -1,76 +1,95 @@
+"""Utilities for key-value format conversions."""
+from __future__ import unicode_literals
+
+import logging
+
+import six
+
+from .oidutil import string_to_text
+
 __all__ = ['seqToKV', 'kvToSeq', 'dictToKV', 'kvToDict']
 
-from openid import oidutil
 
-import types
+_LOGGER = logging.getLogger(__name__)
+
+
+class KVFormError(ValueError):
+    pass
+
 
 def seqToKV(seq, strict=False):
     """Represent a sequence of pairs of strings as newline-terminated
     key:value pairs. The pairs are generated in the order given.
 
     @param seq: The pairs
-    @type seq: [(str, (unicode|str))]
+    @type seq: List[Tuple[six.text_type, six.text_type]], binary_type values are deprecated.
 
     @return: A string representation of the sequence
-    @rtype: str
+    @rtype: six.text_type
     """
     def err(msg):
         formatted = 'seqToKV warning: %s: %r' % (msg, seq)
         if strict:
-            raise ValueError(formatted)
+            raise KVFormError(formatted)
         else:
-            oidutil.log(formatted)
+            _LOGGER.debug(formatted)
 
     lines = []
     for k, v in seq:
-        if isinstance(k, types.StringType):
-            k = k.decode('UTF8')
-        elif not isinstance(k, types.UnicodeType):
-            err('Converting key to string: %r' % k)
-            k = str(k)
+        if not isinstance(k, (six.text_type, six.binary_type)):
+            err('Converting key to text: %r' % k)
+            k = six.text_type(k)
+        if not isinstance(v, (six.text_type, six.binary_type)):
+            err('Converting value to text: %r' % v)
+            v = six.text_type(v)
+
+        k = string_to_text(k, "Binary values for keys are deprecated. Use text input instead.")
+        v = string_to_text(v, "Binary values for values are deprecated. Use text input instead.")
 
         if '\n' in k:
-            raise ValueError(
+            raise KVFormError(
                 'Invalid input for seqToKV: key contains newline: %r' % (k,))
 
         if ':' in k:
-            raise ValueError(
+            raise KVFormError(
                 'Invalid input for seqToKV: key contains colon: %r' % (k,))
 
         if k.strip() != k:
-            err('Key has whitespace at beginning or end: %r' % k)
-
-        if isinstance(v, types.StringType):
-            v = v.decode('UTF8')
-        elif not isinstance(v, types.UnicodeType):
-            err('Converting value to string: %r' % v)
-            v = str(v)
+            err('Key has whitespace at beginning or end: %r' % (k,))
 
         if '\n' in v:
-            raise ValueError(
+            raise KVFormError(
                 'Invalid input for seqToKV: value contains newline: %r' % (v,))
 
         if v.strip() != v:
-            err('Value has whitespace at beginning or end: %r' % v)
+            err('Value has whitespace at beginning or end: %r' % (v,))
 
         lines.append(k + ':' + v + '\n')
 
-    return ''.join(lines).encode('UTF8')
+    return ''.join(lines)
+
 
 def kvToSeq(data, strict=False):
     """
+    Parse newline-terminated key:value pair string into a sequence.
 
     After one parse, seqToKV and kvToSeq are inverses, with no warnings::
 
         seq = kvToSeq(s)
         seqToKV(kvToSeq(seq)) == seq
+
+    @type data: six.text_type, six.binary_type is deprecated
+
+    @rtype: List[Tuple[six.text_type, six.text_type]]
     """
     def err(msg):
         formatted = 'kvToSeq warning: %s: %r' % (msg, data)
         if strict:
-            raise ValueError(formatted)
+            raise KVFormError(formatted)
         else:
-            oidutil.log(formatted)
+            _LOGGER.debug(formatted)
+
+    data = string_to_text(data, "Binary values for data are deprecated. Use text input instead.")
 
     lines = data.split('\n')
     if lines[-1]:
@@ -105,16 +124,17 @@ def kvToSeq(data, strict=False):
                        'whitespace in value %r')
                 err(fmt % (line_num, v))
 
-            pairs.append((k_s.decode('UTF8'), v_s.decode('UTF8')))
+            pairs.append((k_s, v_s))
         else:
             err('Line %d does not contain a colon' % line_num)
 
     return pairs
 
+
 def dictToKV(d):
-    seq = d.items()
-    seq.sort()
+    seq = sorted(d.items())
     return seqToKV(seq)
 
+
 def kvToDict(s):
     return dict(kvToSeq(s))
